@@ -34,8 +34,8 @@ FLAGS:
     -V, --version           Prints version information
 
 OPTIONS:
-        --default-toolchain <default-toolchain>    Choose a default toolchain to install
-        --default-toolchain none                   Do not install any toolchains
+        --default-toolchain <default-toolchain>    Choose a default toolchain
+        --default-toolchain none                   Do not set a default toolchain
 EOF
 }
 
@@ -98,15 +98,13 @@ main() {
     fi
 
     ensure mkdir -p "$_dir"
-    local _latest=$(ensure curl -sSfL -o /dev/null -w %{url_effective} "$ELAN_UPDATE_ROOT/latest" | cut -d'"' -f2 | awk -F/ '{print $NF}')
-
     case "$_arch" in
         *windows*)
-            ensure curl -sSfL "$ELAN_UPDATE_ROOT/download/$_latest/elan-$_arch.zip" > "$_dir/elan-init.zip"
+            ensure curl -sSfL "$ELAN_UPDATE_ROOT/latest/download/elan-$_arch.zip" > "$_dir/elan-init.zip"
             (cd "$_dir"; ensure unzip elan-init.zip; ignore rm elan-init.zip)
             ;;
         *)
-            ensure curl -sSfL "$ELAN_UPDATE_ROOT/download/$_latest/elan-$_arch.tar.gz" > "$_dir/elan-init.tar.gz"
+            ensure curl -sSfL "$ELAN_UPDATE_ROOT/latest/download/elan-$_arch.tar.gz" > "$_dir/elan-init.tar.gz"
             (cd "$_dir"; ensure tar xf elan-init.tar.gz; ignore rm elan-init.tar.gz)
             ;;
     esac
