@@ -1,19 +1,16 @@
 //! Just a dumping ground for cli stuff
 
-use elan::{self, Cfg, Notification, Toolchain, UpdateStatus};
+use crate::errors::*;
+use crate::term2;
+use elan::{Cfg, Notification, Toolchain};
 use elan_dist::dist::ToolchainDesc;
 use elan_utils::notify::NotificationLevel;
 use elan_utils::utils;
-use errors::*;
-use self_update;
-use std;
 use std::io::{BufRead, BufReader, Write};
 use std::path::Path;
 use std::process::{Command, Stdio};
 use std::sync::Arc;
 use std::time::Duration;
-use std::{cmp, iter};
-use term2;
 use wait_timeout::ChildExt;
 
 pub fn confirm(question: &str, default: bool) -> Result<bool> {
@@ -28,7 +25,7 @@ pub fn confirm(question: &str, default: bool) -> Result<bool> {
         _ => false,
     };
 
-    println!("");
+    println!();
 
     Ok(r)
 }
@@ -40,7 +37,7 @@ pub enum Confirm {
 }
 
 pub fn confirm_advanced() -> Result<Confirm> {
-    println!("");
+    println!();
     println!("1) Proceed with installation (default)");
     println!("2) Customize installation");
     println!("3) Cancel installation");
@@ -54,7 +51,7 @@ pub fn confirm_advanced() -> Result<Confirm> {
         _ => Confirm::No,
     };
 
-    println!("");
+    println!();
 
     Ok(r)
 }
@@ -64,7 +61,7 @@ pub fn question_str(question: &str, default: &str) -> Result<String> {
     let _ = std::io::stdout().flush();
     let input = read_line()?;
 
-    println!("");
+    println!();
 
     if input.is_empty() {
         Ok(default.to_string())
@@ -79,7 +76,7 @@ pub fn question_bool(question: &str, default: bool) -> Result<bool> {
     let _ = std::io::stdout().flush();
     let input = read_line()?;
 
-    println!("");
+    println!();
 
     if input.is_empty() {
         Ok(default)
@@ -103,12 +100,12 @@ pub fn read_line() -> Result<String> {
 }
 
 pub fn set_globals(verbose: bool) -> Result<Cfg> {
-    use download_tracker::DownloadTracker;
+    use crate::download_tracker::DownloadTracker;
     use std::cell::RefCell;
 
     let download_tracker = RefCell::new(DownloadTracker::new());
 
-    Ok(Cfg::from_env(Arc::new(move |n: Notification| {
+    Ok(Cfg::from_env(Arc::new(move |n: Notification<'_>| {
         if download_tracker.borrow_mut().handle_notification(&n) {
             return;
         }
@@ -132,107 +129,30 @@ pub fn set_globals(verbose: bool) -> Result<Cfg> {
     }))?)
 }
 
-pub fn show_channel_update(
-    cfg: &Cfg,
-    name: &ToolchainDesc,
-    updated: elan::Result<UpdateStatus>,
-) -> Result<()> {
-    show_channel_updates(cfg, vec![(name.clone(), updated)])
-}
+pub fn show_channel_update(cfg: &Cfg, desc: &ToolchainDesc) -> Result<()> {
+    let toolchain = &cfg.get_toolchain(desc, false).expect("");
+    let version = lean_version(toolchain);
+    let name = desc.to_string();
 
-fn show_channel_updates(
-    cfg: &Cfg,
-    toolchains: Vec<(ToolchainDesc, elan::Result<UpdateStatus>)>,
-) -> Result<()> {
-    let data = toolchains.into_iter().map(|(desc, result)| {
-        let ref toolchain = cfg.get_toolchain(&desc, false).expect("");
-        let version = lean_version(toolchain);
-        let name = desc.to_string();
-
-        let banner;
-        let color;
-        match result {
-            Ok(UpdateStatus::Installed) => {
-                banner = "installed";
-                color = Some(term2::color::BRIGHT_GREEN);
-            }
-            Ok(UpdateStatus::Updated) => {
-                banner = "updated";
-                color = Some(term2::color::BRIGHT_GREEN);
-            }
-            Ok(UpdateStatus::Unchanged) => {
-                banner = "unchanged";
-                color = None;
-            }
-            Err(_) => {
-                banner = "update failed";
-                color = Some(term2::color::BRIGHT_RED);
-            }
-        }
-
-        let width = name.len() + 1 + banner.len();
-
-        (name, banner, width, color, version)
-    });
+    let banner = "installed";
+    let color = Some(term2::color::BRIGHT_GREEN);
 
     let mut t = term2::stdout();
 
-    let data: Vec<_> = data.collect();
-    let max_width = data
-        .iter()
-        .fold(0, |a, &(_, _, width, _, _)| cmp::max(a, width));
-
-    for (name, banner, width, color, version) in data {
-        let padding = max_width - width;
-        let padding: String = iter::repeat(' ').take(padding).collect();
-        let _ = write!(t, "  {}", padding);
-        let _ = t.attr(term2::Attr::Bold);
-        if let Some(color) = color {
-            let _ = t.fg(color);
-        }
-        let _ = write!(t, "{} ", name);
-        let _ = write!(t, "{}", banner);
-        let _ = t.reset();
-        let _ = writeln!(t, " - {}", version);
+    let _ = t.attr(term2::Attr::Bold);
+    if let Some(color) = color {
+        let _ = t.fg(color);
     }
-    let _ = writeln!(t, "");
+    let _ = write!(t, "{} ", name);
+    let _ = write!(t, "{}", banner);
+    let _ = t.reset();
+    let _ = writeln!(t, " - {}", version);
+    let _ = writeln!(t);
 
     Ok(())
 }
 
-pub fn update_all_channels(cfg: &Cfg, self_update: bool, force_update: bool) -> Result<()> {
-    let toolchains = cfg.update_all_channels(force_update)?;
-
-    if toolchains.is_empty() {
-        info!("no updatable toolchains installed.");
-        info!("use 'elan toolchain install' to install a toolchain.")
-    }
-
-    let setup_path = if self_update {
-        self_update::prepare_update()?
-    } else {
-        None
-    };
-
-    if !toolchains.is_empty() {
-        println!("");
-
-        show_channel_updates(cfg, toolchains)?;
-    }
-
-    if let Some(ref setup_path) = setup_path {
-        self_update::run_update(setup_path)?;
-
-        unreachable!(); // update exits on success
-    } else if self_update {
-        // Try again in case we emitted "tool `{}` is already installed" last time.
-        self_update::install_proxies()?;
-    }
-
-    Ok(())
-}
-
-pub fn lean_version(toolchain: &Toolchain) -> String {
+pub fn lean_version(toolchain: &Toolchain<'_>) -> String {
     if toolchain.exists() {
         let lean_path = toolchain.binary_file("lean");
         if utils::is_file(&lean_path) {
@@ -278,34 +198,10 @@ pub fn lean_version(toolchain: &Toolchain) -> String {
             String::from("(lean does not exist)")
         }
     } else {
-        String::from("(toolchain not installed)")
+        String::from("(toolchain will be installed on first use)")
     }
 }
 
-pub fn list_toolchains(cfg: &Cfg) -> Result<()> {
-    let toolchains = cfg.list_toolchains()?;
-
-    if toolchains.is_empty() {
-        println!("no installed toolchains");
-    } else {
-        if let Ok(Some(def_toolchain)) = cfg.find_default() {
-            for toolchain in toolchains {
-                let if_default = if def_toolchain.name() == toolchain.to_string() {
-                    " (default)"
-                } else {
-                    ""
-                };
-                println!("{}{}", &toolchain, if_default);
-            }
-        } else {
-            for toolchain in toolchains {
-                println!("{}", &toolchain);
-            }
-        }
-    }
-    Ok(())
-}
-
 pub fn list_overrides(cfg: &Cfg) -> Result<()> {
     let overrides = cfg.settings_file.with(|s| Ok(s.overrides.clone()))?;
 
@@ -326,7 +222,7 @@ pub fn list_overrides(cfg: &Cfg) -> Result<()> {
             )
         }
         if any_not_exist {
-            println!("");
+            println!();
             info!(
                 "you may remove overrides for non-existent directories with
 `elan override unset --nonexistent`"
@@ -353,10 +249,9 @@ pub fn report_error(e: &Error) {
     if show_backtrace() {
         if let Some(backtrace) = e.backtrace() {
             info!("backtrace:");
-            println!("");
+            println!();
             println!("{:?}", backtrace);
         }
-    } else {
     }
 
     fn show_backtrace() -> bool {
