@@ -3,9 +3,8 @@
 use std::io;
 use std::path::PathBuf;
 
-use elan;
 use elan_dist::{self, temp};
-use elan_utils;
+use error_chain::error_chain;
 
 error_chain! {
     links {
@@ -23,10 +22,6 @@ error_chain! {
         PermissionDenied {
             description("permission denied")
         }
-        ToolchainNotInstalled(t: String) {
-            description("toolchain is not installed")
-            display("toolchain '{}' is not installed", t)
-        }
         InfiniteRecursion {
             description("infinite recursion detected")
         }
