@@ -1,4 +1,4 @@
-pub static ELAN_HELP: &'static str = r"DISCUSSION:
+pub static ELAN_HELP: &str = r"DISCUSSION:
     elan manages your installations of the Lean theorem prover.
     It places `lean` and `lake` binaries in your `PATH` that automatically
     select and, if necessary, download the Lean version described in your
@@ -6,41 +6,32 @@ pub static ELAN_HELP: &'static str = r"DISCUSSION:
     and uninstall Lean versions manually using the commands of the `elan`
     executable.";
 
-pub static SHOW_HELP: &'static str = r"DISCUSSION:
+pub static SHOW_HELP: &str = r"DISCUSSION:
     Shows the name of the active toolchain and the version of `lean`.
 
     If there are multiple toolchains installed then all installed
     toolchains are listed as well.";
 
-pub static UPDATE_HELP: &'static str = r"DISCUSSION:
-    With no toolchain specified, the `update` command updates each of
-    the installed toolchains from the official release channels, then
-    updates elan itself.
-
-    If given a toolchain argument then `update` updates that
-    toolchain, the same as `elan toolchain install`.";
-
-pub static INSTALL_HELP: &'static str = r"DISCUSSION:
+pub static INSTALL_HELP: &str = r"DISCUSSION:
     Installs a specific lean toolchain.
 
     The 'install' command is an alias for 'elan update <toolchain>'.";
 
-pub static DEFAULT_HELP: &'static str = r"DISCUSSION:
-    Sets the default toolchain to the one specified. If the toolchain
-    is not already installed then it is installed first.";
+pub static DEFAULT_HELP: &str = r"DISCUSSION:
+    Sets the default toolchain to the one specified.";
 
-pub static TOOLCHAIN_HELP: &'static str = r"DISCUSSION:
+pub static TOOLCHAIN_HELP: &str = r"DISCUSSION:
     Many `elan` commands deal with *toolchains*, a single
     installation of the Lean theorem prover. `elan` supports multiple
     types of toolchains. The most basic track the official release
-    channels: 'stable' and 'nightly'; but `elan` can also
+    channels: 'stable', 'beta', and 'nightly'; but `elan` can also
     install toolchains from the official archives and from local builds.
 
     Standard release channel toolchain names have the following form:
 
         [<origin>:]<channel>[-<date>]
 
-        <channel>       = stable|nightly|<version>
+        <channel>       = stable|beta|nightly|<version>
         <date>          = YYYY-MM-DD
 
     'channel' is either a named release channel or an explicit version
@@ -55,7 +46,7 @@ pub static TOOLCHAIN_HELP: &'static str = r"DISCUSSION:
     often used to for developing Lean itself. For more information see
     `elan toolchain help link`.";
 
-pub static TOOLCHAIN_LINK_HELP: &'static str = r"DISCUSSION:
+pub static TOOLCHAIN_LINK_HELP: &str = r"DISCUSSION:
     'toolchain' is the custom name to be assigned to the new toolchain.
 
     'path' specifies the directory where the binaries and libraries for
@@ -70,7 +61,18 @@ pub static TOOLCHAIN_LINK_HELP: &'static str = r"DISCUSSION:
     If you now compile a crate in the current directory, the custom
     toolchain 'master' will be used.";
 
-pub static OVERRIDE_HELP: &'static str = r"DISCUSSION:
+pub static TOOLCHAIN_GC_HELP: &str = r"DISCUSSION:
+    Experimental. A toolchain is classified as 'in use' if
+    * it is the default toolchain,
+    * it is registered as an override, or
+    * there is a directory with a `lean-toolchain` file referencing the
+      toolchain and elan has been used in the directory before.
+
+    For safety reasons, the command currently requires passing `--delete`
+    to actually remove toolchains but this may be relaxed in the future
+    when the implementation is deemed stable.";
+
+pub static OVERRIDE_HELP: &str = r"DISCUSSION:
     Overrides configure elan to use a specific toolchain when
     running in a specific directory.
 
@@ -93,14 +95,14 @@ pub static OVERRIDE_HELP: &'static str = r"DISCUSSION:
     override and use the default toolchain again, `elan override
     unset`.";
 
-pub static OVERRIDE_UNSET_HELP: &'static str = r"DISCUSSION:
+pub static OVERRIDE_UNSET_HELP: &str = r"DISCUSSION:
     If `--path` argument is present, removes the override toolchain
     for the specified directory. If `--nonexistent` argument is
     present, removes the override toolchain for all nonexistent
     directories. Otherwise, removes the override toolchain for the
     current directory.";
 
-pub static RUN_HELP: &'static str = r"DISCUSSION:
+pub static RUN_HELP: &str = r"DISCUSSION:
     Configures an environment to use the given toolchain and then runs
     the specified program. The command may be any program, not just
     lean or lake. This can be used for testing arbitrary toolchains
@@ -115,14 +117,14 @@ pub static RUN_HELP: &'static str = r"DISCUSSION:
 
         $ elan run --install nightly lake build";
 
-pub static _DOC_HELP: &'static str = r"DISCUSSION:
+pub static _DOC_HELP: &str = r"DISCUSSION:
     Opens the documentation for the currently active toolchain with
     the default browser.
 
     By default, it opens the documentation index. Use the various
     flags to open specific pieces of documentation.";
 
-pub static COMPLETIONS_HELP: &'static str = r"DISCUSSION:
+pub static COMPLETIONS_HELP: &str = r"DISCUSSION:
     One can generate a completion script for `elan` that is
     compatible with a given shell. The script is output on `stdout`
     allowing one to re-direct the output to the file of their
@@ -223,6 +225,6 @@ pub static COMPLETIONS_HELP: &'static str = r"DISCUSSION:
 
         PS C:\> elan completions powershell >> %USERPROFILE%\Documents\WindowsPowerShell\Microsoft.PowerShell_profile.ps1";
 
-pub static TOOLCHAIN_ARG_HELP: &'static str = "Toolchain name, such as 'stable', 'nightly', \
-     or '3.3.0'. For more information see `elan \
+pub static TOOLCHAIN_ARG_HELP: &str = "Toolchain name, such as 'stable', 'beta', 'nightly', \
+     or '4.3.0'. For more information see `elan \
      help toolchain`";
