@@ -1,7 +1,7 @@
+use crate::common;
+use crate::errors::*;
+use crate::self_update::{self, InstallOpts};
 use clap::{App, AppSettings, Arg};
-use common;
-use errors::*;
-use self_update::{self, InstallOpts};
 use std::env;
 
 pub fn main() -> Result<()> {
@@ -32,7 +32,7 @@ pub fn main() -> Result<()> {
             Arg::with_name("default-toolchain")
                 .long("default-toolchain")
                 .takes_value(true)
-                .help("Choose a default toolchain to install"),
+                .help("Choose a default toolchain"),
         )
         .arg(
             Arg::with_name("no-modify-path")
@@ -43,12 +43,12 @@ pub fn main() -> Result<()> {
     let matches = cli.get_matches();
     let no_prompt = matches.is_present("no-prompt");
     let verbose = matches.is_present("verbose");
-    let default_toolchain = matches.value_of("default-toolchain").unwrap_or("nightly");
+    let default_toolchain = matches.value_of("default-toolchain").unwrap_or("stable");
     let no_modify_path = matches.is_present("no-modify-path");
 
     let opts = InstallOpts {
         default_toolchain: default_toolchain.to_owned(),
-        no_modify_path: no_modify_path,
+        no_modify_path,
     };
 
     self_update::install(no_prompt, verbose, opts)?;
