@@ -2,24 +2,19 @@
 //! for installing from a directory or tarball to an installation
 //! prefix, represented by a `Components` instance.
 
-extern crate filetime;
-extern crate flate2;
-extern crate zstd;
-extern crate tar;
-
-use errors::*;
-use temp;
+use crate::errors::*;
 
 use std::fs::{self, File};
 use std::io::{self, Read, Seek};
 use std::path::{Path, PathBuf};
 
+use time::OffsetDateTime;
 use zip::ZipArchive;
 
 #[derive(Debug)]
-pub struct TarPackage<'a>(temp::Dir<'a>);
+pub struct TarPackage();
 
-impl<'a> TarPackage<'a> {
+impl TarPackage {
     pub fn unpack<R: Read>(stream: R, path: &Path) -> Result<()> {
         let mut archive = tar::Archive::new(stream);
         // The lean-installer packages unpack to a directory called
@@ -62,9 +57,9 @@ fn unpack_without_first_dir<R: Read>(archive: &mut tar::Archive<R>, path: &Path)
 }
 
 #[derive(Debug)]
-pub struct ZipPackage<'a>(temp::Dir<'a>);
+pub struct ZipPackage();
 
-impl<'a> ZipPackage<'a> {
+impl ZipPackage {
     pub fn unpack<R: Read + Seek>(stream: R, path: &Path) -> Result<()> {
         let mut archive = ZipArchive::new(stream).chain_err(|| ErrorKind::ExtractingPackage)?;
         /*
@@ -122,8 +117,11 @@ impl<'a> ZipPackage<'a> {
                     }
                 }
             } // make sure to close `dst` before setting mtime
-            let mtime = entry.last_modified().to_time()?.unix_timestamp_nanos();
-            let mtime = filetime::FileTime::from_unix_time((mtime / 1000000000) as i64 , (mtime % 1000000000) as u32);
+            let mtime = OffsetDateTime::try_from(entry.last_modified().unwrap_or_else(zip::DateTime::default_for_write))?.unix_timestamp_nanos();
+            let mtime = filetime::FileTime::from_unix_time(
+                (mtime / 1000000000) as i64,
+                (mtime % 1000000000) as u32,
+            );
             filetime::set_file_times(&full_path, mtime, mtime).unwrap();
         }
 
@@ -132,9 +130,9 @@ impl<'a> ZipPackage<'a> {
 }
 
 #[derive(Debug)]
-pub struct TarGzPackage<'a>(TarPackage<'a>);
+pub struct TarGzPackage();
 
-impl<'a> TarGzPackage<'a> {
+impl TarGzPackage {
     pub fn unpack<R: Read>(stream: R, path: &Path) -> Result<()> {
         let stream = flate2::read::GzDecoder::new(stream);
 
@@ -147,9 +145,9 @@ impl<'a> TarGzPackage<'a> {
 }
 
 #[derive(Debug)]
-pub struct TarZstdPackage<'a>(TarPackage<'a>);
+pub struct TarZstdPackage();
 
-impl<'a> TarZstdPackage<'a> {
+impl TarZstdPackage {
     pub fn unpack<R: Read>(stream: R, path: &Path) -> Result<()> {
         let stream = zstd::stream::read::Decoder::new(stream)?;
 
